function varargout = imageinfo(varargin)
% Provides information regarding the opened image

% What I learned about programming this GUI:
%
% The aim of this GUI is to provide information to the user given the input image in the variable "varargin"
% The result of this GUI is to return to the interface the following two pieced of information, which may be
% changed by the user or could just be the default values (as shown in brackets):
%   (a) The number of principal components to use in the analysis (either 10 or the number of channels, whichever is less)
%   (b) If the image is an RGB image (yes, if there are three channels, otherwise no)
% 
% The GUI returns back to the main interface after the user presses the "X" or clicks "Continue ...."
% Since we use the UIWAIT function, we must put the UIRESUME function is somewhere, at the point where we 
% want to return control back to the calling function (MACCMIA) in this case.  This UIRESUME function should
% also be included in the CLOSEREQ function, which is specified when editing the GUI with GUIDE.
% 
% The output arguments are returned with VARARGOUT{:} type commands.
%
% KGD: 19 September 2002
% KGD: Updated: 6 August 2004 so that just the image dimensions and type are passed to it, not the entire image.


if (nargin == 0) | (nargin == 5)  % LAUNCH GUI if nargin is 0 or 3
   if nargin == 0
      error('This function is intended to be called by MACC MIA only, or please provide the correct input.');
   else
      fig = openfig(mfilename,'reuse');
      set(fig,'Color',get(0,'defaultUicontrolBackgroundColor'));  % Use system color scheme for figure:
      handles = guihandles(fig);  % Generate a structure of handles to pass to callbacks, and store it. 
      
      handles.nRows   = varargin{1};
      handles.nCols   = varargin{2};
      handles.nLayers = varargin{3};
      handles.imType  = varargin{4};
      guidata(fig, handles);
      
      set(handles.xAxisPixels,'String',[num2str(handles.nCols),' pixels']);
      set(handles.yAxisPixels,'String',[num2str(handles.nRows),' pixels']);
      set(handles.NumChannels,'String',num2str(handles.nLayers));	
      if handles.nLayers > 10
        set(handles.NumPCToUse,'String','10');
      else
        set(handles.NumPCToUse,'String',num2str(handles.nLayers));      
      end;
   end;
   if handles.nLayers == 3
       if strcmp(handles.imType,'double')
           set(handles.ThisIsRGBImage,'enable','off');
           set(handles.ThisIsRGBImage,'value',0.0);
       elseif strcmp(handles.imType,'uint8')
           set(handles.ThisIsRGBImage,'enable','on');
           set(handles.ThisIsRGBImage,'value',1.0);
       end;
   else 
      set(handles.ThisIsRGBImage,'enable','off');
      set(handles.ThisIsRGBImage,'value',0.0);
   end;
   guidata(fig, handles);    % Update the handles with new information  
   uiwait(fig); 

   
   handles = guidata(fig);
   delete(fig);                              % Delete the figure window, we will just pass the info back and then quit   
   
   if ~isfield(handles,'NumPC')              % Implies the number of required principal components was not changed ...
      varargout{1} = min(10,handles.nLayers);% use the defaults in that case = min(10,NumChannels) 
   else
      varargout{1} = handles.NumPC;          % otherwise, use the user specified number of PC's
   end;
   
   
   if ~isfield(handles,'IsRGB')              % Implies the RGB setting (yes or no) was not changed from the default
      if handles.nLayers == 3                % use RGB=1 if NumChannels is 3, else use IsRGB=0 
         varargout{2} = 1;
      else
         varargout{2} = 0;
      end;
   else
      varargout{2} = handles.IsRGB;     
   end;
     

elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK
	try
		if (nargout)
			[varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
		else
			feval(varargin{:}); % FEVAL switchyard
		end
	catch
		disp(lasterr);
	end
end;



% --------------------------------------------------------------------
function varargout = NumPCToUse_Callback(h, eventdata, handles, varargin)
NumPCEnteredByUser = ceil(str2double(get(handles.NumPCToUse,'string')));
if (isnan(NumPCEnteredByUser)) | (NumPCEnteredByUser < 1)
   errordlg({'Please enter a positive, integer value'},'Input error','modal');
   set(handles.NumPCToUse,'String',get(handles.NumChannels,'String'));   
   return;
elseif NumPCEnteredByUser > str2double(get(handles.NumChannels,'String'))
   errordlg({'The number of principal component used cannot', ...
             'exceed the number of channels in the image.'},'Input error','modal');
   set(handles.NumPCToUse,'String',get(handles.NumChannels,'String'));                
   return;
elseif (  (NumPCEnteredByUser < 3) & (strcmp(handles.imType,'double'))  )
    set(handles.NumPCToUse,'String',get(handles.NumChannels,'String'));
    return;
else
   handles.NumPC = NumPCEnteredByUser;
   set(handles.NumPCToUse,'String',num2str(NumPCEnteredByUser));
   handles.InformationChanged = 1;  
   %  if NumPCEnteredByUser == 3    % maybe the user wishes to consider this an RGB image
   %     set(handles.ThisIsRGBImage,'enable','on');
   %     set(handles.ThisIsRGBImage,'value',1.000);
   %  end;
   guidata(h, handles);    % Update the handles with new information  
end;


% --------------------------------------------------------------------
function varargout = ThisIsRGBImage_Callback(h, eventdata, handles, varargin)
if get(handles.ThisIsRGBImage,'value')
  handles.IsRGB = 1;
else
  handles.IsRGB = 0;
end;
handles.InformationChanged = 1;
guidata(h, handles);    % Update the handles 

% --------------------------------------------------------------------
function varargout = HelpImageInfo_Callback(h, eventdata, handles, varargin)
msgbox({'This window provides information about the data file',...
        'that was opened.',' ', ...
        'The displayed information shows the number', ...
        'of pixels in the image width (x-axis), the number of ',...
        'pixels in the image height (y-axis) and the number of ', ...
        'channels, also called wavelengths.',' ',...
        'If there are three channels, it is probably an RGB image,', ...
        'otherwise it could be a multispectral image.',' ', ...
        'The default is to use as many principal components as there', ...
        'are channels, but a lower number may be used, if required.'}, ...
        'Also, if there are more than 10 channels, then it is probably', ...
        'not necessary to have more than 10 principal components',...
        'Help - Image Information','help','modal');


% --------------------------------------------------------------------
function varargout = OKToContinue_Callback(h, eventdata, handles, varargin)
uiresume(handles.ImageInfoGUI);